#[cfg(test)]
use fake::{faker::internet::en::SafeEmail, faker::name::en::Name, Dummy};

use sqlx::PgPool;

use crate::utils::validation::{email_is_valid, AsyncValidate, AsyncValidateError, DbContext};

use super::{Area, Function, Role};

#[derive(Debug)]
#[cfg_attr(test, derive(Dummy))]
pub struct UserChangeset {
    #[cfg_attr(test, dummy(faker = "Name()"))]
    pub name: String,
    #[cfg_attr(test, dummy(faker = "SafeEmail()"))]
    pub email: String,
    #[cfg_attr(test, dummy(expr = "Role::Staff"))]
    pub role: Role,
    #[cfg_attr(test, dummy(expr = "vec![Function::Posten]"))]
    pub functions: Vec<Function>,
    #[cfg_attr(test, dummy(expr = "1"))]
    pub area_id: i32,
}

impl <'a>AsyncValidate<'a> for UserChangeset {
    type Context = DbContext<'a>;

    async fn validate_with_context(&self, context: &'a Self::Context) -> Result<(), AsyncValidateError> {
        email_is_valid(&self.email)?;
        area_exists(context.pool, self.area_id).await?;

        Ok(())
    }
}

async fn area_exists(pool: &PgPool, id: i32) -> Result<(), AsyncValidateError> {
    if Area::read_by_id(pool, id).await?.is_none() {
        return Err(AsyncValidateError::new(
            "Angegebener Bereich für Nutzer existiert nicht!",
        ));
    }

    Ok(())
}
